import requests
import pandas as pd
from datetime import date
from bs4 import BeautifulSoup as bs


# 업종별 시세 페이지 호출
def get_upjong_list_P():
    #requests를 통한 호출
    url = 'https://finance.naver.com/sise/sise_group.naver?type=upjong'
    response = requests.get(url)

    df = pd.read_html(response.text)[0]
    df = df.dropna()
    df = df.reset_index(drop=True)

    cols = ['업종명', '전일대비', '전체', '상승', '보합', '하락', '등락그래프']
    df.columns = cols
    df = df.drop('등락그래프', axis=1)
    df[['전체', '상승', '보합', '하락']] = df[['전체', '상승', '보합', '하락']].astype(int)

    html = bs(response.text, 'html.parser')
    link_list = html.select('td > a')
    link_list = link_list[:-5]
    temp = []

    for i in link_list :
        temp.append(i['href'].split("=")[2])

    # 업종코드 추가
    df["업종코드"] = temp
    return df


# 각 업종별 URL 추출 함수
def get_url(upjong_cd):
    '''
    1. 기본 URL 생성
    2. 파라미터를 통해 URL 변경
    3. return URL
    '''
    URL = 'https://finance.naver.com/sise/sise_group_detail.naver?type=upjong&no='
    return URL + upjong_cd


# 각 업종별 페이지 스크래핑 함수

def get_scrap_upjongcd(upjong_cd):
    '''
    1. URL 생성
    2. request 요청
    3. response를 데이터프레임으로 생성
    4. return 데이터프레임
    '''

    # requests를 통한 호출
    url = get_url(upjong_cd)
    response = requests.get(url)

    df = pd.read_html(response.text)[2]
    df = df.dropna(how='all')
    df = df.dropna(how='all')
    df = df.dropna(axis=1)
    df = df.reset_index(drop=True)
    cols = ['종목명', '현재가', '전일비', '등락률', '매수호가', '매도호가', '거래량', '거래대금', '전일거래량']
    df.columns = cols

    # 업종별 호출
    # 79rows
    return df


# 결과값 csv 파일로 저장
def save_file_csv(df, code=0):
    '''
    1. 파일명 설정
    2. 파일 저장
    3. 파일 확인
    '''

    today = date.today()
    dt = today.strftime("%y%m%d")

    # 업종리스트, 업종별에 따라 파일명 설정
    if (code == 0) & (df.columns[0] == '업종명'):
        file_name = f"industry_{dt}.csv"
    elif (code != 0) & (df.columns[0] == '종목명'):
        file_name = f"sector{code}_{dt}.csv"
    else:
        file_name = f"noname_{dt}.csv"

    # 파일 저장
    df.to_csv(file_name, index=False)

    # 파일 확인
    try:
        display(pd.read_csv(file_name))
    except:
        pass



import time
import requests
import pandas as pd
from tqdm import trange
from konlpy.tag import Okt  # 형태소분석기 : Openkoreatext
from tqdm.notebook import tqdm
import matplotlib.pyplot as plt  # 시각화
from collections import Counter  # 빈도 수 세기
from bs4 import BeautifulSoup as bs
from wordcloud import WordCloud, STOPWORDS  # wordcloud 만들기


font_path = './HANBATANG.TTF'
tqdm.pandas()
requests.packages.urllib3.disable_warnings(requests.packages.urllib3.exceptions.InsecureRequestWarning)


def fn_page_scrapping(date, page_no):
    # 1) 날짜, page 번호로 url 만들기 
    url = "https://finance.naver.com/news/news_list.naver?mode=RANK"
    params = f"mode=RANK&date={date}&page={page_no}"

    # 2) requests.post() 로 요청
    headers = {"user-agent": "Mozilla/5.0"}
    response = requests.post(url, params=params, headers=headers, verify=False)
   
    # 3) bs 적용
    html = bs(response.text, 'html.parser')

    # 4) 테이블 찾기
    df = html.select("#contentarea_left > div > ul > li > ul > li > a")
    page_list = pd.DataFrame(df)
    page_list.columns = ['기사 제목']

    # 5) a 태그 목록 찾기
    a_href = [a["href"] for a in html.select("#contentarea_left > div > ul > li > ul > li > a")]

    # 6) 내용링크에 a 태그 주소 추가
    page_list["내용링크"] = a_href

    # 7) 데이터 프레임 변환
    return page_list


def fn_get_content(link):
    # 1) 수집할 URL 만들기    
    base_url = "https://finance.naver.com"
    url = base_url + link

    # 2) requests로 HTTP 요청하기  
    response = requests.get(url, verify=False)

    # 3) response.text에 BeautifulSoup 적용하기
    html = bs(response.text, 'html.parser')

    # 4) 내용 가져오기    
    content = html.find_all("div", {"id": "content"})[0].text

    # 5) time.sleep()
    time.sleep(0.01)

    # 6) 내용 반환하기
    return content[1:-230]


def fn_most_viewed_news(date):
    page_list = []

    url = "https://finance.naver.com/news/news_list.naver?mode=RANK" 
    params = f"mode=RANK&date={date}"
    headers = {"user-agent": "Mozilla/5.0"}
    response = requests.post(url, params=params, headers=headers, verify=False)
    html = bs(response.text, 'html.parser')

    last_page = int(html.select("table > tr > td.pgRR > a")[-1]["href"].split("=")[-1])

    for page_no in trange(1, last_page+1):
        result = fn_page_scrapping(date, page_no)
        page_list.append(result)
        time.sleep(0.01)

    pages_list = pd.concat(page_list)
    pages_list = pages_list.reset_index(drop=True)

    contents = []

    for link in pages_list['내용링크']:
        contents.append(fn_get_content(link))

    pages_list['내용'] = contents
    
    return pages_list


def wc1_get_noun(date):
    # 스크래핑 출력물 저장
    news = fn_most_viewed_news(date)

    script = news['내용']
    script.to_csv('word.txt', encoding='utf-8')
    text = open('word.txt', encoding='utf-8').read()
    
    okt = Okt()
    
    return [word for word in okt.nouns(text) if len(word) > 1]


# 중복 단어 배제, wordcloud 만들기
def wc1_make_wc(date):
    
    # 함수 출력 값 할당 해주기 
    noun = wc1_get_noun(date)
    
    count = Counter(noun)
    word = dict(count.most_common(200))

    stopwords = set(STOPWORDS)

    wc = WordCloud(max_font_size=200,
                   background_color="white",
                   width=2000, height=500,
                   stopwords=stopwords).generate_from_frequencies(word)
    
    plt.figure(figsize=(40, 40))
    plt.imshow(wc)
    plt.tight_layout(pad=0)
    plt.axis('off')

    return plt.show()


# ### 워드 클라우드2. 기사 [기사 제목]으로 워드 클라우드 만들어 보기

# In[27]:


# 형태소 분석기를 통해 명사만 추출하기 1글자 명사는 제외
def wc2_get_noun(date):
    
    # 스크래핑 출력물 저장
    news = fn_most_viewed_news(date)
    
    script = news['기사 제목']
    script.to_csv('word2.txt', encoding='utf-8')
    text = open('word2.txt', encoding='utf-8').read()
    
    okt = Okt()
    
    return [word for word in okt.nouns(text) if len(word) > 1]


# 중복 단어 배제, wordcloud 만들기
def wc2_make_wc(date):
    
    # 함수 출력 값 할당 해주기 
    noun = wc2_get_noun(date)
    
    count = Counter(noun)
    word = dict(count.most_common(200))

    stopwords = set(STOPWORDS)

    wc = WordCloud(max_font_size=200,
                   background_color="white",
                   width=2000, height=500,
                   stopwords=stopwords).generate_from_frequencies(word)
    
    plt.figure(figsize=(40, 40))
    plt.imshow(wc)
    plt.tight_layout(pad=0)
    plt.axis('off')

    return plt.show()



import time
import requests
import pandas as pd
from datetime import datetime
import matplotlib.pyplot as plt
from bs4 import BeautifulSoup as bs


# 업종별 순위(rank_upjong()), 요약정보(get_summary()), 세부정보get_detail(), 베스트5워스트5(best5(), worst5_())
# 를 스크래핑 하는 클래스
# 클래스의 목적 : 주식의 정보를 그룹별로 파악하기 쉽게 볼 수 있도록 하는 것
class Upjong:
    def __init__(self, upjong_nm, upjong_num):
        self.upjong_nm = upjong_nm
        self.upjong_num = upjong_num
        plt.rcParams['font.family'] = 'Malgun Gothic'

    @staticmethod
    def get_page_sise_by_upjong():
        '''
        업종별 시세 페이지를 스크래핑하는 함수
        '''
        url = 'https://finance.naver.com/sise/sise_group.naver?type=upjong'
        response = requests.get(url)
        df = pd.read_html(response.text)[0]
        df = df.dropna()
        df = df.reset_index(drop=True)
        cols = ['업종명', '전일대비', '전체', '상승', '보합', '하락', '등락그래프']
        df.columns = cols
        df = df.drop('등락그래프', axis=1)
        df[['전체', '상승', '보합', '하락']] = df[['전체', '상승', '보합', '하락']].astype(int)

        html = bs(response.text, 'html.parser')
        upjong_no = [link['href'].split('&')[-1].replace('no=','') for link in html.select('td > a')[:-5]]
        df['업종번호'] = upjong_no

        today_date = datetime.today().strftime('%Y-%m-%d')
        file_name = f'업종별시세-{today_date}.csv'
        df.to_csv(f'data/{file_name}', index=False)
        df = pd.read_csv(f'data/{file_name}', dtype={'업종번호': object})
        return df

    def rank_upjong(self):
        right_now = datetime.today().strftime('%Y-%m-%d %X')
        sise_by_upjong = Upjong.get_page_sise_by_upjong()
        total_upjong = sise_by_upjong.shape[0]
        today_rank = sise_by_upjong[sise_by_upjong['업종명'] == self.upjong_nm].index[0] + 1
        print(f'{self.upjong_nm}의 {right_now} 현재 전일대비 등락률 순위는 {total_upjong}종목중 {today_rank}위 입니다!')
    
    def get_summary(self):
        right_now = datetime.today().strftime('%Y-%m-%d %X')
        before_summary = Upjong.get_page_sise_by_upjong()
        summary = before_summary[before_summary['업종명'] == self.upjong_nm]
        print(f'{right_now} 현재 {self.upjong_nm} 업종의 요약정보입니다!')
        
        plt.figure(figsize=(6, 6))
        ratio = [summary.iloc[0, 3], summary.iloc[0, 4], summary.iloc[0, 5]]
        labels = ['상승', '보합', '하락']
        explode = [0.40, 0.20, 0.00]
        colors = ['red', 'grey', 'blue']
        patches, texts, autotexts = plt.pie(ratio, explode=explode, colors=colors, autopct='%.1f%%', textprops={'size': 9})
        plt.legend(labels)
        
        for t in autotexts:
            t.set_color("white")
            t.set_fontsize(12)
        plt.show()
        
        plt.show()
        
        return summary
    
    def get_detail(self):
        url_for_detail = 'https://finance.naver.com/sise/sise_group_detail.naver?type=upjong'
        sub_url = f'{url_for_detail}&no={self.upjong_num}'
        response_detail = requests.get(sub_url)
        
        df_for_detail = Upjong.get_page_sise_by_upjong()
        df_detail = pd.read_html(response_detail.text)[-1]
        df_detail = df_detail.dropna(how='all', axis=1)
        df_detail = df_detail.dropna(how='all')
        df_detail['업종명'] = df_for_detail[df_for_detail['업종번호'] == self.upjong_num]['업종명'].iloc[0]
        df_detail = df_detail.reset_index(drop=True)
        cols = [
                '업종명', '종목명', '현재가', '전일비',
                '등락률', '거래량', '거래대금' ,'전일거래량',
                '매수호가', '매도호가'
                ]
        df_detail = df_detail[cols]
        
        cols_int = ['현재가', '전일비', '거래량', '거래대금', '전일거래량', '매수호가', '매도호가']
        df_detail[cols_int] = df_detail[cols_int].astype(int)
        return df_detail
    
    def best5(self):
        right_now = datetime.today().strftime('%Y-%m-%d %X')
        best5 = Upjong.get_detail(self).head(5)
        if best5.shape[0] < 5:
            print('해당 업종 내 기업이 5개 미만 입니다!')
            print(f'{right_now} 현재 {self.upjong_nm} 업종의 모든 기업정보입니다!')
            return best5
        else:
            print(f'{right_now} 현재 {self.upjong_nm} 업종의 전일대비 등락률 best5기업입니다!')
            
            # 막대그래프
            plt.figure(figsize=(9, 6))
            x = best5['종목명']
            y = best5['현재가']
            plt.xlabel('종목명')
            plt.ylabel('현재가')

            plt.bar(x, y, color='red')
            for i, v in enumerate(x):
                plt.text(v, y[i], f'{i+1}위 : ' + best5['등락률'][i],
                         fontsize=12,
                         color="red",
                         horizontalalignment='center',
                         verticalalignment='bottom')

            plt.show()
            return best5
    
    def worst5(self):
        right_now = datetime.today().strftime('%Y-%m-%d %X')
        worst5 = Upjong.get_detail(self)
        r_idx = [i for i in range(worst5.shape[0]-1, -1, -1)]
        worst5 = pd.DataFrame(worst5, index=r_idx).reset_index(drop=True)
        worst5 = worst5.head(5)
        if worst5.shape[0] < 5:
            print('해당 업종 내 기업이 5개 미만 입니다!')
            print(f'{right_now} 현재 {self.upjong_nm} 업종의 모든 기업정보입니다!')
            
            return worst5
        else:
            print(f'{right_now} 현재 {self.upjong_nm} 업종의 전일대비 등락률 worst5기업입니다!')

            # 막대그래프
            plt.figure(figsize=(9, 6))
            x = worst5['종목명']
            y = worst5['현재가']
            plt.xlabel('종목명')
            plt.ylabel('현재가')

            plt.bar(x, y, color = 'blue')
            for i, v in enumerate(x):
                plt.text(v, y[i], f'{i+1}위 : ' + worst5['등락률'][i],
                         fontsize=12,
                         color="blue",
                         horizontalalignment='center',
                         verticalalignment='bottom')

            plt.show()
            
            return worst5


# 테마별 순위(rank_theme()), 요약정보(get_summary()), 세부정보get_detail(), 베스트5워스트5(best5(), worst5())
# 를 스크래핑 하는 클래스
# 클래스의 목적 : 주식의 정보를 테마별로 파악하기 쉽게 볼 수 있도록 하는 것
class Theme:
    def __init__(self, theme_nm, theme_num):
        self.theme_nm = theme_nm
        self.theme_num = theme_num
        plt.rcParams['font.family'] = 'Malgun Gothic'

    @staticmethod
    def get_pages_sise_by_theme():
        '''
        1. 끝 페이지를 번호를 얻기위한 기본 url로 끝페이지 정보 얻기
        2. 반복문으로 첫페이지부터 끝페이지까지 스크래핑
        3. 스크래핑 한 데이터 저장 후 불러오기
        '''
        url_for_end = 'https://finance.naver.com/sise/theme.naver'
        response = requests.get(url_for_end)
        html = bs(response.text, 'html.parser')
        end_page = int(html.select('td.pgRR > a')[0]['href'][-1])

        pages_by_theme = []
        for i in range(1, end_page+1):
            url = f'https://finance.naver.com/sise/theme.naver?&page={i}'
            # 요청
            response = requests.get(url)
            page = pd.read_html(response.text)[0]

            # 결측치 제거
            page = page.dropna()
            page = page.reset_index(drop=True)

            # 이중컬럼 제거
            cols = ['테마명', '전일대비', '최근3일등락률(평균)', '상승', '보합', '하락', '주도주1', '주도주2']
            page.columns = cols

            #  테마내 전체 기업 수를 알기 위해 전체 컬럼 추가
            page['전체'] = page['상승'] + page['보합'] + page['하락']

            # 컬럼 순서 정리

            cols = ['테마명', '전일대비', '최근3일등락률(평균)', '전체', '상승', '보합', '하락', '주도주1', '주도주2']
            page = page[cols]

            # 상승, 보합, 하락 컬럼 데이터타입 정수형으로 변환
            page[['전체', '상승', '보합', '하락']] = page[['전체', '상승', '보합', '하락']].astype(int)

            # 테마번호 컬럼 추가
            html = bs(response.text, 'html.parser')
            theme_num = []
            for link in html.select('tr > td > a')[:-15][::3]:
                theme_num.append(link['href'].split('=')[-1])

            page['테마번호'] = theme_num

            pages_by_theme.append(page)

            time.sleep(0.001)
        pages_by_theme = pd.concat(pages_by_theme).reset_index(drop=True)    

        today_date = datetime.today().strftime('%Y-%m-%d')
        file_name = f'테마별시세-{today_date}.csv'
        pages_by_theme.to_csv(f'data/{file_name}', index=False)
        pages_by_theme = pd.read_csv(f'data/{file_name}', dtype={'테마번호':object})

        return pages_by_theme
    
    def rank_theme(self):
        right_now = datetime.today().strftime('%Y-%m-%d %X')
        sise_by_theme = Theme.get_pages_sise_by_theme()
        total_theme = sise_by_theme.shape[0]
        today_rank = sise_by_theme[sise_by_theme['테마명'] == self.theme_nm].index[0] + 1
        print(f'{self.theme_nm}의 {right_now} 현재 전일대비 등락률 순위는 {total_theme}종목중 {today_rank}위 입니다!')

    def get_summary(self):
        right_now = datetime.today().strftime('%Y-%m-%d %X')
        before_summary = Theme.get_pages_sise_by_theme()
        summary = before_summary[before_summary['테마명'] == self.theme_nm]
        print(f'{right_now} 현재 {self.theme_nm} 테마의 요약정보입니다!')
        
        # 파이차트
        plt.figure(figsize=(6, 6))
        ratio = [summary.iloc[0, 4], summary.iloc[0, 5], summary.iloc[0, 6]]
        labels = ['상승', '보합', '하락']
        explode = [0.40, 0.20, 0.00]
        colors = ['red', 'grey', 'blue']
        patches, texts, autotexts = plt.pie(ratio, explode=explode, colors=colors, autopct='%.1f%%', textprops={'size': 9})
        plt.legend(labels)
        for t in autotexts:
            t.set_color("white")
            t.set_fontsize(12)
        plt.show()
        
        return summary

    def get_detail(self):
        base_url = 'https://finance.naver.com/sise/sise_group_detail.naver?type=theme&no='
        sub_url = f'{base_url}{self.theme_num}'
        response_detail = requests.get(sub_url)
        
        df_for_detail = Theme.get_pages_sise_by_theme()
        df_detail = pd.read_html(response_detail.text)[-1]
        df_detail = df_detail.dropna(how='all', axis=1)
        df_detail = df_detail.dropna(how='all')
        df_detail['테마명'] = df_for_detail[df_for_detail['테마번호'] == self.theme_num]['테마명'].iloc[0]
        df_detail = df_detail.reset_index(drop=True)
        cols = [
                '테마명', '종목명', '현재가', '전일비',
                '등락률', '거래량', '거래대금','전일거래량',
                '매수호가', '매도호가'
                ]
        df_detail = df_detail[cols]
        
        cols_int = ['현재가', '전일비', '거래량', '거래대금', '전일거래량', '매수호가', '매도호가']
        df_detail[cols_int] = df_detail[cols_int].astype(int)
        return df_detail
        
    def best5(self):
        right_now = datetime.today().strftime('%Y-%m-%d %X')
        best5 = Theme.get_detail(self).head(5)
        if best5.shape[0] < 5:
            print('해당 테마 내 기업이 5개 미만 입니다!')
            print(f'{right_now} 현재 {self.theme_nm} 테마의 모든 기업정보입니다!')
            return best5
        else:
            print(f'{right_now} 현재 {self.theme_nm} 테마의 전일대비 등락률 best5기업입니다!')
            
            # 막대그래프
            plt.figure(figsize=(9, 6))
            x = best5['종목명']
            y = best5['현재가']
            plt.xlabel('종목명')
            plt.ylabel('현재가')

            plt.bar(x, y, color = 'red')
            for i, v in enumerate(x):
                plt.text(v, y[i], f'{i+1}위 : ' + best5['등락률'][i],
                         fontsize=12,
                         color="red",
                         horizontalalignment='center',
                         verticalalignment='bottom')

            plt.show()
            
            return best5
    
    def worst5(self):
        right_now = datetime.today().strftime('%Y-%m-%d %X')
        worst5= Theme.get_detail(self)
        r_idx = [i for i in range(worst5.shape[0]-1, -1, -1)]
        worst5 = pd.DataFrame(worst5, index=r_idx).reset_index(drop=True)
        worst5 = worst5.head(5)
        if worst5.shape[0] < 5:
            print('해당 테마 내 기업이 5개 미만 입니다!')
            print(f'{right_now} 현재 {self.theme_nm} 테마의 모든 기업정보입니다!')
            
            return worst5
        else:
            print(f'{right_now} 현재 {self.theme_nm} 테마의 전일대비 등락률 worst5기업입니다!')
            
            # 막대그래프
            plt.figure(figsize=(9, 6))
            x = worst5['종목명']
            y = worst5['현재가']
            plt.xlabel('종목명')
            plt.ylabel('현재가')

            plt.bar(x, y, color = 'blue')
            for i, v in enumerate(x):
                plt.text(v, y[i], f'{i+1}위 : ' + worst5['등락률'][i],
                         fontsize=12,
                         color="blue",
                         horizontalalignment='center',
                         verticalalignment='bottom')

            plt.show()
            
            return worst5

        
# 그룹사별 순위(rank_groupsa()), 요약정보(get_summary()), 세부정보get_detail(), 베스트3워스트3(best3(), worst3())
# 를 스크래핑 하는 클래스
# 클래스의 목적 : 주식의 정보를 그룹사별로 파악하기 쉽게 볼 수 있도록 하는 것
class Groupsa:
    def __init__(self, group_nm, group_num):
        self.group_nm = group_nm
        self.group_num = group_num
        plt.rcParams['font.family'] = 'Malgun Gothic'

    @staticmethod
    def get_page_sise_by_group():
        '''
        그룹사별 시세 페이지를 스크래핑하는 함수
        '''
        url = 'https://finance.naver.com/sise/sise_group.naver?type=group'
        response = requests.get(url)
        df = pd.read_html(response.text)[0]
        df = df.dropna()
        df = df.reset_index(drop=True)
        cols = ['그룹명', '전일대비', '전체', '상승', '보합', '하락', '등락그래프']
        df.columns = cols
        df = df.drop('등락그래프', axis=1)
        df[['전체', '상승', '보합', '하락']] = df[['전체', '상승', '보합', '하락']].astype(int)

        html = bs(response.text, 'html.parser')
        group_no = [link['href'].split('&')[-1].replace('no=','') for link in html.select('td > a')[:-5]]
        df['그룹번호'] = group_no

        today_date = datetime.today().strftime('%Y-%m-%d')
        file_name = f'그룹별시세-{today_date}.csv'
        df.to_csv(f'data/{file_name}', index=False)
        df = pd.read_csv(f'data/{file_name}', dtype={'그룹번호': object})
        return df

    def rank_groupsa(self):
        right_now = datetime.today().strftime('%Y-%m-%d %X')
        sise_by_group = Groupsa.get_page_sise_by_group()
        total_group = sise_by_group.shape[0]
        today_rank = sise_by_group[sise_by_group['그룹명'] == self.group_nm].index[0] + 1
        print(f'{self.group_nm}의 {right_now} 현재 전일대비 등락률 순위는 {total_group}그룹중 {today_rank}위 입니다!')
    
    def get_summary(self):
        right_now = datetime.today().strftime('%Y-%m-%d %X')
        before_summary = Groupsa.get_page_sise_by_group()
        summary = before_summary[before_summary['그룹명'] == self.group_nm]
        print(f'{right_now} 현재 {self.group_nm} 그룹의 요약정보입니다!')
        
        plt.figure(figsize=(6, 6))
        ratio = [summary.iloc[0, 3], summary.iloc[0, 4], summary.iloc[0, 5]]
        labels = ['상승', '보합', '하락']
        explode = [0.40, 0.20, 0.00]
        colors = ['red', 'grey', 'blue']
        patches, texts, autotexts = plt.pie(ratio, explode=explode, colors=colors, autopct='%.1f%%', textprops={'size': 9})
        plt.legend(labels)
        
        for t in autotexts:
            t.set_color("white")
            t.set_fontsize(12)
        plt.show()
        
        plt.show()
        
        return summary
    
    def get_detail(self):
        url_for_detail = 'https://finance.naver.com/sise/sise_group_detail.naver?type=group'
        sub_url = f'{url_for_detail}&no={self.group_num}'
        response_detail = requests.get(sub_url)
        
        df_for_detail = Groupsa.get_page_sise_by_group()
        df_detail = pd.read_html(response_detail.text)[-1]
        df_detail = df_detail.dropna(how='all', axis=1)
        df_detail = df_detail.dropna(how='all')
        df_detail['그룹명'] = df_for_detail[df_for_detail['그룹번호'] == self.group_num]['그룹명'].iloc[0]
        df_detail = df_detail.reset_index(drop=True)
        cols = [
                '그룹명', '종목명', '현재가', '전일비',
                '등락률', '거래량', '거래대금','전일거래량',
                '매수호가', '매도호가'
                ]
        df_detail = df_detail[cols]
        
        cols_int = ['현재가', '전일비', '거래량', '거래대금', '전일거래량', '매수호가', '매도호가']
        df_detail[cols_int] = df_detail[cols_int].astype(int)
        return df_detail
    
    def best3(self):
        right_now = datetime.today().strftime('%Y-%m-%d %X')
        best3 = Groupsa.get_detail(self).head(3)
        if best3.shape[0] < 3:
            print('해당 그룹 내 기업이 3개 미만 입니다!')
            print(f'{right_now} 현재 {self.group_nm} 그룹의 모든 기업정보입니다!')
            return best3
        else:
            print(f'{right_now} 현재 {self.group_nm} 그룹의 전일대비 등락률 best3기업입니다!')
            
            # 막대그래프
            plt.figure(figsize=(9, 6))
            x = best3['종목명']
            y = best3['현재가']
            plt.xlabel('종목명')
            plt.ylabel('현재가')

            plt.bar(x, y, color = 'red')
            for i, v in enumerate(x):
                plt.text(v, y[i], f'{i+1}위 : ' + best3['등락률'][i],
                         fontsize=12,
                         color="red",
                         horizontalalignment='center',
                         verticalalignment='bottom')

            plt.show()
            return best3
    
    def worst3(self):
        right_now = datetime.today().strftime('%Y-%m-%d %X')
        worst3= Groupsa.get_detail(self)
        r_idx = [i for i in range(worst3.shape[0]-1, -1, -1)]
        worst3 = pd.DataFrame(worst3, index=r_idx).reset_index(drop=True)
        worst3 = worst3.head(3)
        if worst3.shape[0] < 3:
            print('해당 그룹 내 기업이 3개 미만 입니다!')
            print(f'{right_now} 현재 {self.group_nm} 그룹의 모든 기업정보입니다!')
            
            return worst3
        else:
            print(f'{right_now} 현재 {self.group_nm} 그룹의 전일대비 등락률 worst3기업입니다!')
            
            # 막대그래프
            plt.figure(figsize=(9, 6))
            x = worst3['종목명']
            y = worst3['현재가']
            plt.xlabel('종목명')
            plt.ylabel('현재가')

            plt.bar(x, y, color = 'blue')
            for i, v in enumerate(x):
                plt.text(v, y[i], f'{i+1}위 : ' + worst3['등락률'][i],
                         fontsize=12,
                         color="blue",
                         horizontalalignment='center',
                         verticalalignment='bottom')

            plt.show()
            
            return worst3



from .Jun import *
from .Yong import *
from .KimJuhee import *
from .ParkJiHye import *
from .KimHeeYeon import *
from .YangHeeKwon import *

__all__ = [
    'fn_page_scrapping',
    'fn_get_content',
    'fn_most_viewed_news',
    'wc1_get_noun',
    'wc1_make_wc',
    'wc2_get_noun',
    'wc2_make_wc',
    'Upjong',
    'Theme',
    'Groupsa',
    'get_top_stock',
    'get_stock_sise',
    'get_upjong_list_P',
    'get_url',
    'get_scrap_upjongcd',
    'save_file_csv',
    'get_upjong_list_K',
    'jongmok_df',
    'jongmok_list',
    'get_theme_list',
    'get_item',
    'ControlTower',
]



import time
import requests
import pandas as pd
from bs4 import BeautifulSoup as bs


# 업종 코드 호출
def get_upjong_list_K():
    #requests를 통한 호출
    url = 'https://finance.naver.com/sise/sise_group.naver?type=upjong'
    response = requests.get(url)

    df = pd.read_html(response.text)[0]
    df = df.dropna()
    df = df.reset_index(drop=True)

    cols = ['upjong_name', '전일대비', '전체', '상승', '보합', '하락', '등락그래프']
    df.columns = cols
    df = df.drop('등락그래프', axis=1)
    df[['전체', '상승', '보합', '하락']] = df[['전체', '상승', '보합', '하락']].astype(int)

    html = bs(response.text, 'html.parser')
    link_list = html.select('td > a')
    link_list = link_list[:-5]
    temp = []

    for i in link_list :
        temp.append(i['href'].split("=")[2])

    # 업종코드 추가
    df['upjong_code'] = temp
    df = df[['upjong_name', 'upjong_code']]
    return df


# 업종코드를 통한 종목 관련 전체 df
def jongmok_df(upjong_cd):
    sel_url = f'https://finance.naver.com/sise/sise_group_detail.naver?type=upjong&no={upjong_cd}'
    sel_respones = requests.get(sel_url)
    table = pd.read_html(sel_respones.text)[2]

    #결측치 제거
    temp = table.drop(['Unnamed: 11', 'Unnamed: 10', '토론실'], axis = 1)
    temp = temp.drop([0])[0:-2]

    #업종코드 컬럼 추가 및 정렬
    temp['upjong_cd'] = upjong_cd
    temp = temp[['upjong_cd', '종목명', '현재가', '전일비', '등락률', '매수호가', '매도호가', '거래량', '거래대금','전일거래량']]
    temp.columns = ['upjong_cd', 'jongmok_name', '현재가', '전일비', '등락률', '매수호가', '매도호가', '거래량', '거래대금','전일거래량']
    return temp


# 업종코드와 종목이름, 종목코드
def jongmok_list():
    jongmok_lists = []
    for i in get_upjong_list_K()['upjong_code'].to_list():
        url_for_detail = 'https://finance.naver.com/sise/sise_group_detail.naver?type=upjong'
        sub_url = f'{url_for_detail}&no={i}'
        response_detail = requests.get(sub_url)
        df_detail = pd.read_html(response_detail.text)[-1]
        df_detail = df_detail.dropna(how='all', axis=1)
        df_detail = df_detail.dropna(how='all')
        df_detail['upjong_code'] = i
        html = bs(response_detail.text, 'html.parser')
        jongmok_codes = html.select('table > tbody > tr > td > div > a')
        jongmok_code_list = []
        for jongmok_code in jongmok_codes:
            jongmok_code = jongmok_code['href'].split('=')[-1]
            jongmok_code_list.append(jongmok_code)
        df_detail['jongmok_code'] = jongmok_code_list
        cols = [
                'upjong_code', '종목명', '현재가', '전일비',
                '등락률', '거래량', '거래대금' ,'전일거래량',
                '매수호가', '매도호가', 'jongmok_code'
                ]
        df_detail = df_detail[cols]
        df_detail = df_detail.rename(columns={'종목명':'jongmok_name'})
        df_detail = df_detail[['upjong_code', 'jongmok_name', 'jongmok_code']]
        jongmok_lists.append(df_detail)
    jongmok_df = pd.concat(jongmok_lists).reset_index(drop = True)
    return jongmok_df


def get_theme_list():
    '''
    1. 끝 페이지를 번호를 얻기위한 기본 url로 끝페이지 정보 얻기
    2. 반복문으로 첫페이지부터 끝페이지까지 스크래핑
    3. 스크래핑 한 데이터 저장 후 불러오기
    '''
    url_for_end = 'https://finance.naver.com/sise/theme.naver'
    response = requests.get(url_for_end)
    html = bs(response.text, 'html.parser')
    end_page = int(html.select('td.pgRR > a')[0]['href'][-1])

    pages_by_theme = []
    for i in range(1, end_page + 1):
        url = f'https://finance.naver.com/sise/theme.naver?&page={i}'
        # 요청
        response = requests.get(url)
        page = pd.read_html(response.text)[0]

        # 결측치 제거
        page = page.dropna()
        page = page.reset_index(drop=True)

        # 이중컬럼 제거
        cols = ['테마명', '전일대비', '최근3일등락률(평균)', '상승', '보합', '하락', '주도주1', '주도주2']
        page.columns = cols

        # 테마번호 컬럼 추가
        html = bs(response.text, 'html.parser')
        theme_num = []
        for link in html.select('tr > td > a')[:-15][::3]:
            theme_num.append(link['href'].split('=')[-1])

        page['테마번호'] = theme_num

        pages_by_theme.append(page)
        time.sleep(0.001)
    pages_by_theme = pd.concat(pages_by_theme).reset_index(drop=True)
    pages_by_theme = pages_by_theme.rename(columns={'테마명': 'theme_name'})
    pages_by_theme = pages_by_theme.rename(columns={'테마번호': 'theme_code'})
    pages_by_theme = pages_by_theme[['theme_name', 'theme_code']]
    return pages_by_theme



from setuptools import setup, find_packages

setup(
    name='shasha4',
    version='0.0.9',
    description='For Scraping Naver Finance',
    url='https://github.com/MoonsRainbow/shasha4.git',
    author='MoonsRainbow',
    author_email='Moons_Rainbow@kakao.com',
    license='MoonsRainbow',
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'requests',
        'tqdm',
        'bs4',
        'pandas',
        'konlpy',
        'wordcloud',
        'matplotlib'
    ]
)



# 필요한 모듈 불러오기
import requests
import pandas as pd
from tqdm.notebook import tqdm
from bs4 import BeautifulSoup as bs

tqdm.pandas()
pd.set_option('display.max_row', 500)
pd.set_option('display.max_columns', 150)


# 실시간 검색어 순위 가져오기
def get_top_stock():
    # url 생성
    url = 'https://finance.naver.com/sise/lastsearch2.naver'

    #response 받기
    headers = {'user-agent' : 'Mozilla/5.0'}
    response = requests.get(url, headers=headers)

    # 데이터 프레임 생성
    df_top_stocks = pd.read_html(response.text)[1]
    df_top_stocks = df_top_stocks.dropna(how='all')
    df_top_stocks = df_top_stocks.reset_index(drop=True)

    # 코드를 찾아서 추가하기
    find_tag = bs(response.text, 'html.parser')
    code = []
    for i in find_tag.select('td > a'):
        code.append(i["href"].split("=")[-1])

    # 불필요 데이터 제거 / 추후에 데이터를 추가 예정
    df_top_stocks["업종코드"] = code
    df_top_stocks = df_top_stocks.drop("전일비", axis=1)
    df_top_stocks = df_top_stocks.drop("등락률", axis=1)
    df_top_stocks = df_top_stocks.drop("거래량", axis=1)
    df_top_stocks = df_top_stocks.drop("현재가", axis=1)
    df_top_stocks = df_top_stocks.drop("시가", axis=1)
    df_top_stocks = df_top_stocks.drop("고가", axis=1)
    df_top_stocks = df_top_stocks.drop("저가", axis=1)
    df_top_stocks = df_top_stocks.drop("PER", axis=1)
    df_top_stocks = df_top_stocks.drop("ROE", axis=1)
    df_top_stocks = df_top_stocks[["순위", "업종코드", "종목명", "검색비율"]]

    return df_top_stocks


# 종목코드를 이용하여 시세 데이터 가져오기
def get_stock_sise(stock_code):
    # url 생성
    stock_url = f"https://finance.naver.com/item/sise.naver?code={stock_code}"
    headers = {'user-agent' : 'Mozilla/5.0'}
    
    #response 받기
    response2 = requests.get(stock_url, headers=headers)
    
    # 테이블 생성
    juyo_sise = pd.read_html(response2.text)[1]
    juyo_sise = juyo_sise.dropna()
    juyo_sise = juyo_sise.reset_index(drop=True)
    
    # 테이블 형태 변경
    t1 = juyo_sise[[0, 1]].set_index(0).T
    t2 = juyo_sise[[2, 3]].set_index(2).T
    t2.index = t1.index
    juyo_sise = pd.concat([t1, t2], axis=1)
   
    return juyo_sise



import time
import platform
import pandas as pd
from matplotlib import rc
from requests import request
from multiprocessing import Pool
from bs4 import BeautifulSoup as bs
from matplotlib import pyplot as plt


# IPython 환경이 아닐 경우, 에러 발생.
try:
    from IPython.display import clear_output
except:
    # 에러 발생 시 패스.
    pass


# 전역 변수 선언.
HEADERS = {
    'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36'
}


# 멀티 프로세스 처리 함수
def get_item(code):
    """
    주식 종목의 코드를 입력받고, 주식 종목의 정보를 스크래핑 해옵니다.

    :param code: str, 주식 종목 코드.
    :return: pandas.DataFrame, 주식 종목 정보.
    """
    item_auto_cols = {
        'cd': '종목 코드',
        'nm': '종목 명',
        'mt': '마켓 타입',
        'ms': '마켓 상태',
        'tyn': '거래 가능 여부',
        'aa': '거래 대금',
        'aq': '거래 량',
        'nv': '현재 가격',
        'cr': '등락률',
        'cv': '등락가',
        'hv': '최고가',
        'lv': '최저가',
        'ul': '상한가',
        'll': '하한가',
        'pcv': '전일 종가',
        'sv': '전일가',
        'ov': '시가',
        'bps': 'BPS',
        'eps': 'EPS'
    }
    _uri = ('get', f'https://polling.finance.naver.com/api/realtime?query=SERVICE_ITEM:{code}')
    _headers = {'User-Agent': 'Mozilla/5.0 (Macintosh;'}
    resp = request(*_uri, headers=_headers)
    datas = resp.json()['result']['areas'][0]['datas'][0]
    datas = {v: datas[k] for k, v in item_auto_cols.items()}
    return pd.DataFrame(datas.values(), index=datas.keys()).T


class ControlTower:
    """
    스크래핑 해온 주식 종목들을 관리합니다.
    """

    def __init__(self):
        """
        생성자 메서드
        """
        self.__upjong_uri = ('get', 'https://finance.naver.com/sise/sise_group.naver?type=upjong')
        self.__items_uri = 'https://finance.naver.com/sise/sise_group_detail.naver?type=upjong&no={}'
        self.__upjong = self.get_upjong()
        self.__upjong_items = dict()
        self.__plot_items = dict()

    @property
    def upjong(self):
        """:return: 업종 정보."""
        return self.__upjong

    @property
    def upjong_items(self):
        """:return: 업종에 해당하는 주식 종목들."""
        return self.__upjong_items

    @property
    def plot_items(self):
        """:return: 그래프 생성에 필요한 정보들."""
        return self.__plot_items

    def get_upjong(self):
        """
        업종 정보들을 스크래핑 해옵니다.

        :return: pd.DataFrame, 업종 정보.
        """
        resp = request(*self.__upjong_uri, headers=HEADERS)
        df = pd.read_html(resp.text)[0]
        df.columns = [col[1] for col in list(df.columns)]
        df = df.dropna(axis=0, how='all')
        df = df.reset_index(drop=True)
        df = df.astype({'전체': int, '상승': int, '보합': int, '하락': int})
        html = bs(resp.text, 'html.parser')
        df['업종 코드'] = [tag_a['href'].split('=')[-1] for tag_a in html.find('table', {'class': 'type_1'}).find_all('a')]
        return df

    def get_upjong_items(self, upjong_code):
        """
        업종에 해당하는 주식 종목의 코드들을 스크래핑 해옵니다.

        :param upjong_code: str, 업종 코드.
        :return: dict, 업종에 해당하는 종목 명, 종목 코드 딕셔너리.
        """
        if upjong_code not in self.__upjong_items:
            resp = request('get', self.__items_uri.format(upjong_code), headers=HEADERS)
            self.__upjong_items[upjong_code] = {tag_a.text: tag_a['href'].split('=')[-1] for tag_a in bs(resp.text, 'html.parser').select('div.name_area > a')}
        return self.__upjong_items[upjong_code]

    def get_items_auto(self, upjong_code):
        """
        주식 종목들의 정보를 8초 단위로 스크래핑 해옵니다.

        스크래핑 해온 정보들을 저장하고, 데이터프레임으로 만들어 디스플레이합니다.

        :param upjong_code: str, 업종 코드.
        """
        print('데이터를 불러오는 중 입니다...')
        for i in range(10):
            pool = Pool(processes=len(self.__upjong_items[upjong_code]))
            item_auto_datas = pool.map(get_item, list(self.__upjong_items[upjong_code].values()))
            for auto_data in item_auto_datas:
                now_code = auto_data['종목 코드'][0]
                if now_code not in self.__plot_items:
                    self.__plot_items[now_code] = {
                        'price': [auto_data['현재 가격'][0]],
                        'volume': [0],
                        'name': auto_data['종목 명'][0],
                        'prev_volume': auto_data['거래 량'][0]
                    }
                else:
                    plot_item = self.__plot_items[now_code]
                    plot_item['price'].append(auto_data['현재 가격'][0])
                    plot_item['volume'].append(auto_data['거래 량'][0] - plot_item['prev_volume'])
                    plot_item['prev_volume'] = auto_data['거래 량'][0]
            clear_output()
            try:
                display(pd.concat(item_auto_datas, ignore_index=True))
            except:
                print('N')
            time.sleep(8)

    def show_plot(self, item_code):
        """
        수집된 정보들로 8초봉 차트를 그립니다.

        :param item_code: str, 주식 종목 코드.
        """
        if item_code not in self.__plot_items:
            print('수집하지 않은 코드입니다.')
        else:
            if platform.system() == 'Darwin':
                rc('font', family='AppleGothic')
            else:
                plt.rcParams['font.family'] = 'Malgun Gothic'
            item = self.__plot_items[item_code]
            price_chart = plt.subplot(2, 1, 1)
            price_chart.title.set_text(item['name'])
            plt.plot(item['price'], 'o-')
            plt.ylabel('Price')
            plt.xticks(range(0, len(item['price'])))
            plt.xticks(visible=False)
            plt.subplot(2, 1, 2, sharex=price_chart)
            plt.bar(range(0, len(item['price'])), item['volume'])
            plt.xlabel('Per 8 Sec')
            plt.ylabel('Volume')
            plt.show()
